@@ -1,7 +1,7 @@
-/* $OpenLDAP: pkg/ldap/libraries/libldap/search.c,v 1.76.2.11 2010/04/14 18:08:23 quanah Exp $ */
+/* $OpenLDAP: pkg/ldap/libraries/libldap/search.c,v 1.76.2.14 2011/01/04 23:50:05 kurt Exp $ */
 /* This work is part of OpenLDAP Software <http://www.openldap.org/>.
  *
- * Copyright 1998-2010 The OpenLDAP Foundation.
+ * Copyright 1998-2011 The OpenLDAP Foundation.
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -182,6 +182,11 @@ ldap_pvt_search_s(
 
 	if( rc <= 0 ) {
 		/* error(-1) or timeout(0) */
+		if ( ld->ld_errno == LDAP_TIMEOUT ) {
+			/* cleanup request */
+			(void) ldap_abandon( ld, msgid );
+			ld->ld_errno = LDAP_TIMEOUT;
+		}
 		return( ld->ld_errno );
 	}
 
@@ -302,7 +307,7 @@ ldap_build_search_req(
 	if ( LDAP_IS_UDP(ld) ) {
 		struct sockaddr sa = {0};
 		/* dummy, filled with ldo_peer in request.c */
-	    err = ber_write( ber, &sa, sizeof( sa ), 0 );
+	    err = ber_write( ber, (char *) &sa, sizeof( sa ), 0 );
 	}
 	if ( LDAP_IS_UDP(ld) && ld->ld_options.ldo_version == LDAP_VERSION2) {
 	    char *dn = ld->ld_options.ldo_cldapdn;
