@@ -1,8 +1,8 @@
 /* getpeereid.c */
-/* $OpenLDAP: pkg/ldap/libraries/liblutil/getpeereid.c,v 1.24.2.3 2008/02/11 23:26:42 kurt Exp $ */
+/* $OpenLDAP: pkg/ldap/libraries/liblutil/getpeereid.c,v 1.24.2.8 2011/01/04 23:50:10 kurt Exp $ */
 /* This work is part of OpenLDAP Software <http://www.openldap.org/>.
  *
- * Copyright 2000-2008 The OpenLDAP Foundation.
+ * Copyright 2000-2011 The OpenLDAP Foundation.
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -14,6 +14,10 @@
  * <http://www.OpenLDAP.org/license.html>.
  */
 
+#ifndef _GNU_SOURCE
+#define _GNU_SOURCE 1			/* Needed for glibc struct ucred */
+#endif
+
 #include "portable.h"
 
 #ifndef HAVE_GETPEEREID
@@ -58,6 +62,7 @@ int lutil_getpeereid( int s, uid_t *euid, gid_t *egid
 		*euid = ucred_geteuid( uc );
 		*egid = ucred_getegid( uc );
 		ucred_free( uc );
+		return 0;
 	}
 
 #elif defined( SO_PEERCRED )
