@@ -1,7 +1,7 @@
-/* $OpenLDAP: pkg/ldap/servers/slapd/back-monitor/proto-back-monitor.h,v 1.33.2.5 2008/02/11 23:26:47 kurt Exp $ */
+/* $OpenLDAP: pkg/ldap/servers/slapd/back-monitor/proto-back-monitor.h,v 1.33.2.10 2011/01/13 19:02:46 quanah Exp $ */
 /* This work is part of OpenLDAP Software <http://www.openldap.org/>.
  *
- * Copyright 2001-2008 The OpenLDAP Foundation.
+ * Copyright 2001-2011 The OpenLDAP Foundation.
  * Portions Copyright 2001-2003 Pierangelo Masarati.
  * All rights reserved.
  *
@@ -77,6 +77,12 @@ extern int
 monitor_cache_destroy LDAP_P((
 	monitor_info_t		*mi ));
 
+extern int
+monitor_back_release(
+	Operation *op,
+	Entry *e,
+	int rw );
+
 /*
  * connections
  */
@@ -152,20 +158,24 @@ monitor_back_register_overlay_info LDAP_P((
 	slap_overinst		*on ));
 extern int
 monitor_back_register_overlay LDAP_P((
-	BackendDB		*be ));
+	BackendDB		*be,
+	struct slap_overinst	*on,
+	struct berval		*ndn_out ));
 extern int
 monitor_back_register_backend_limbo LDAP_P((
 	BackendInfo		*bi ));
 extern int
 monitor_back_register_database_limbo LDAP_P((
 	BackendDB		*be,
-	struct berval	*ndn ));
+	struct berval		*ndn_out ));
 extern int
 monitor_back_register_overlay_info_limbo LDAP_P((
 	slap_overinst		*on ));
 extern int
 monitor_back_register_overlay_limbo LDAP_P((
-	BackendDB		*be ));
+	BackendDB		*be,
+	struct slap_overinst	*on,
+	struct berval		*ndn_out ));
 extern monitor_subsys_t *
 monitor_back_get_subsys LDAP_P((
 	const char		*name ));
