@@ -1,7 +1,7 @@
-/* $OpenLDAP: pkg/ldap/servers/slapd/modify.c,v 1.276.2.12 2009/03/05 18:16:29 quanah Exp $ */
+/* $OpenLDAP: pkg/ldap/servers/slapd/modify.c,v 1.276.2.18 2011/01/04 23:50:21 kurt Exp $ */
 /* This work is part of OpenLDAP Software <http://www.openldap.org/>.
  *
- * Copyright 1998-2009 The OpenLDAP Foundation.
+ * Copyright 1998-2011 The OpenLDAP Foundation.
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -82,12 +82,14 @@ do_modify(
 	if ( rs->sr_err != LDAP_SUCCESS ) {
 		Debug( LDAP_DEBUG_ANY, "%s do_modify: slap_parse_modlist failed err=%d msg=%s\n",
 			op->o_log_prefix, rs->sr_err, rs->sr_text );
+		send_ldap_result( op, rs );
 		goto cleanup;
 	}
 
 	if( get_ctrls( op, rs, 1 ) != LDAP_SUCCESS ) {
 		Debug( LDAP_DEBUG_ANY, "%s do_modify: get_ctrls failed\n",
 			op->o_log_prefix, 0, 0 );
+		/* get_ctrls has sent results.	Now clean up. */
 		goto cleanup;
 	}
 
@@ -682,7 +684,7 @@ slap_sort_vals(
 	AttributeDescription *ad;
 	MatchingRule *mr;
 	int istack[sizeof(int)*16];
-	int i, j, k, l, ir, jstack, match, *ix, itmp, nvals, rc;
+	int i, j, k, l, ir, jstack, match, *ix, itmp, nvals, rc = LDAP_SUCCESS;
 	int is_norm;
 	struct berval a, *cv;
 
@@ -690,7 +692,7 @@ slap_sort_vals(
 #define	SWAP(a,b,tmp)	tmp=(a);(a)=(b);(b)=tmp
 #define	COMP(a,b)	match=0; rc = ordered_value_match( &match, \
 						ad, mr, SLAP_MR_EQUALITY \
-								| SLAP_MR_VALUE_OF_ATTRIBUTE_SYNTAX \
+								| SLAP_MR_VALUE_OF_ASSERTION_SYNTAX \
 								| SLAP_MR_ASSERTED_VALUE_NORMALIZED_MATCH \
 								| SLAP_MR_ATTRIBUTE_VALUE_NORMALIZED_MATCH, \
 								&(a), &(b), text );
@@ -703,6 +705,8 @@ slap_sort_vals(
 
 	ad = ml->sml_desc;
 	nvals = ml->sml_numvals;
+	if ( nvals <= 1 )
+		goto ret;
 
 	/* For Modifications, sml_nvalues is NULL if normalization wasn't needed.
 	 * For Attributes, sml_nvalues == sml_values when normalization isn't needed.
@@ -832,15 +836,14 @@ slap_sort_vals(
 
 	slap_sl_free( ix, ctx );
 
-	if ( rc != LDAP_SUCCESS ) {
-		return rc;
-	} else if ( match == 0 ) {
+	if ( rc == LDAP_SUCCESS && match == 0 ) {
 		/* value exists already */
 		assert( i >= 0 );
 		assert( i < nvals );
-		return LDAP_TYPE_OR_VALUE_EXISTS;
+		rc = LDAP_TYPE_OR_VALUE_EXISTS;
 	}
-	return LDAP_SUCCESS;
+ ret:
+	return rc;
 }
 
 /* Enter with bv->bv_len = sizeof buffer, returns with
@@ -848,21 +851,11 @@ slap_sort_vals(
  */
 void slap_timestamp( time_t *tm, struct berval *bv )
 {
-	struct tm *ltm;
-#ifdef HAVE_GMTIME_R
-	struct tm ltm_buf;
+	struct tm ltm;
 
-	ltm = gmtime_r( tm, &ltm_buf );
-#else
-	ldap_pvt_thread_mutex_lock( &gmtime_mutex );
-	ltm = gmtime( tm );
-#endif
+	ldap_pvt_gmtime( tm, &ltm );
 
-	bv->bv_len = lutil_gentime( bv->bv_val, bv->bv_len, ltm );
-
-#ifndef HAVE_GMTIME_R
-	ldap_pvt_thread_mutex_unlock( &gmtime_mutex );
-#endif
+	bv->bv_len = lutil_gentime( bv->bv_val, bv->bv_len, &ltm );
 }
 
 /* Called for all modify and modrdn ops. If the current op was replicated
@@ -876,7 +869,7 @@ void slap_mods_opattrs(
 	struct berval name, timestamp, csn = BER_BVNULL;
 	struct berval nname;
 	char timebuf[ LDAP_LUTIL_GENTIME_BUFSIZE ];
-	char csnbuf[ LDAP_LUTIL_CSNSTR_BUFSIZE ];
+	char csnbuf[ LDAP_PVT_CSNSTR_BUFSIZE ];
 	Modifications *mod, **modtail, *modlast;
 	int gotcsn = 0, gotmname = 0, gotmtime = 0;
 
