@@ -1,8 +1,8 @@
 /* collect.c - Demonstration of overlay code */
-/* $OpenLDAP: pkg/ldap/servers/slapd/overlays/collect.c,v 1.5.2.10 2009/06/19 21:51:22 quanah Exp $ */
+/* $OpenLDAP: pkg/ldap/servers/slapd/overlays/collect.c,v 1.5.2.15 2011/01/28 18:53:37 quanah Exp $ */
 /* This work is part of OpenLDAP Software <http://www.openldap.org/>.
  *
- * Copyright 2003-2009 The OpenLDAP Foundation.
+ * Copyright 2003-2011 The OpenLDAP Foundation.
  * Portions Copyright 2003 Howard Chu.
  * All rights reserved.
  *
@@ -386,11 +386,7 @@ collect_response( Operation *op, SlapReply *rs )
 			* don't modify it directly. Make a copy and
 			* work with that instead.
 			*/
-			if ( !( rs->sr_flags & REP_ENTRY_MODIFIABLE )) {
-				rs->sr_entry = entry_dup( rs->sr_entry );
-				rs->sr_flags |= REP_ENTRY_MODIFIABLE |
-					REP_ENTRY_MUSTBEFREED;
-			}
+			rs_entry2modifiable( op, rs, on );
 
 			/* Loop for each attribute in this collectinfo */
 			for(idx=0; idx<ci->ci_ad_num; idx++) {
